package com.gic.enterprise.config;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.exception.CommonException;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.context.annotation.Configuration;

import javax.validation.ConstraintViolationException;

/**
 * 采集GIC系统操作日志
 * 
 * @author leeon
 * @date   2019年6月28日
 */
@Aspect
@Configuration
public class ParamAop {

    private Logger logger = LogManager.getLogger(ParamAop.class);

    /**
     * 环绕通知
     *
     * @param  joinPoint
     * @return
     * @throws Throwable
     */
    @Around(value = "execution(* com.gic.*.service..*(..))")
    public Object around(ProceedingJoinPoint joinPoint) throws Throwable {
        try {
            // 当前方法
            String currentMethodName = getMethodName(joinPoint);
            // 当前方法参数列表
            Object[] args = joinPoint.getArgs();
            logger.info("请求方法:{}, 参数:{}", currentMethodName, args);
            // 执行当前方法
            return joinPoint.proceed();
        } catch (Throwable throwable) {
            logger.warn("拦截器错误", throwable);
            return handlerException(throwable);
        }
    }

    private ServiceResponse handlerException(Throwable e) {
        ServiceResponse response = new ServiceResponse();
        if (e instanceof CommonException) {
            response.setCode(((CommonException) e).getErrorCode());
        } else if (e instanceof ConstraintViolationException) {
            response.setCode(ErrorCode.SYSTEM_ERROR.getCode());
        } else {
            response.setCode(ErrorCode.SYSTEM_ERROR.getCode());

        }
        response.setMessage(e.getMessage());
        return response;
    }

    /**
     * 获取当前方法
     *
     * @param  joinPoint
     * @return
     * @throws
     */
    private String getMethodName(ProceedingJoinPoint joinPoint) throws NoSuchMethodException {
        Signature signature = joinPoint.getSignature();
        MethodSignature methodSignature = (MethodSignature) signature;
        Object target = joinPoint.getTarget();
        return target.getClass().getName() + "." + methodSignature.getName();
    }
}