package com.gic.enterprise.web.controller.union;

import com.gic.api.base.commons.Page;
import com.gic.auth.dto.*;
import com.gic.auth.service.*;
import com.gic.download.utils.OperationResultUtils;
import com.gic.enterprise.dto.union.UnionEnterpriseAuthDTO;
import com.gic.enterprise.dto.union.UnionEnterpriseAuthResDTO;
import com.gic.enterprise.exception.CommonException;
import com.gic.enterprise.service.UnionEnterpriseAuthApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.web.vo.union.*;
import com.gic.open.api.dto.ApplicationDTO;
import com.gic.open.api.service.ApplicationApiService;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.utils.CreateRandomUtils;
import com.gic.enterprise.utils.UserDetail;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.utils.AuthCodeExpireTimeUtils;
import com.gic.marketing.process.api.service.sms.SmsSendApiService;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("union-enterprise")
public class UnionEnterpriseAuthController {
    private static Logger logger = LogManager.getLogger(UnionEnterpriseAuthController.class);

    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private SmsSendApiService smsSendApiService;
    @Autowired
    private AuthCodeApiService authCodeApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private ResourceGroupApiService resourceGroupApiService;
    @Autowired
    private AuditedGroupApiService auditedGroupApiService;
    @Autowired
    private AccountGroupApiService accountGroupApiService;
    @Autowired
    private ApplicationApiService applicationApiService;
    @Autowired
    private UnionEnterpriseAuthApiService unionEnterpriseAuthApiService;

    private static final String UNION_ENTERPRISE_AUTH_SMS_CODE = "GICSHHT001";

    /**
     * 商户名称查询商户是否存在
     * @param enterpriseName
     * @return
     */
    @RequestMapping("check-enterprise-by-name")
    public RestResponse checkEnterpriseName(String enterpriseName) {
        EnterpriseDTO enterpriseDTO = checkEnterpriseByEnterpriseName(enterpriseName);
        if (enterpriseDTO == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "联合商户不存在或者是自有商户");
        }
        return RestResponse.success(EntityUtil.changeEntityNew(EnterpriseInfoVO.class, enterpriseDTO));
    }

    @RequestMapping("send-auth-code-to-union-enterprise")
    public RestResponse sendAuthCode(String enterpriseName) {
        EnterpriseDTO unionEnterprise = checkEnterpriseByEnterpriseName(enterpriseName);
        if (unionEnterprise == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "联合商户不存在或者是自有商户");
        }
        ServiceResponse<UserDTO> unionUserRes = userApiService.getUserByEnterpriseId(unionEnterprise.getEnterpriseId());
        UserDTO unionUser = unionUserRes.getResult();

        UserDetail userDetail = UserDetailUtils.getUserDetail();

        //验证码生成
        AuthCodeDTO authCodeDTO = new AuthCodeDTO();
        authCodeDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        //生成4位数字验证码
        String authCode = CreateRandomUtils.getStringRandom(4);
        authCodeDTO.setAuthCode(authCode);
        authCodeDTO.setExpirationTime(AuthCodeExpireTimeUtils.setAuthCodeExpireTime(5));
        ServiceResponse<Integer> result = authCodeApiService.saveAuth(authCodeDTO);
        if (result.isSuccess()) {
            Integer authCodeId = result.getResult();
            //发送手机验证码
            String[] smsArr = new String[] { unionEnterprise.getEnterpriseName(),
                    userDetail.getEnterpriseInfo().getEnterpriseName(),
                    authCode,
                    userDetail.getEnterpriseInfo().getEnterpriseName()};
            ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms(UNION_ENTERPRISE_AUTH_SMS_CODE,
                    unionEnterprise.getEnterpriseId(),
                    unionUser.getPhoneAreaCode(),
                    unionUser.getPhoneNumber(),
                    smsArr);
            if (!smsSendResult.isSuccess()) {
                return EnterpriseRestResponse.failure(smsSendResult);
            } else {
                return RestResponse.success(authCodeId);
            }
        }
        return EnterpriseRestResponse.failure(result);
    }

    /**
     * 发送验证码校验联合商户，如果已存在则进入资源授权页面；如果不存在，则新建联合商户
     * @param authCode
     * @param authCodeId
     * @param enterpriseName
     * @return
     */
    @RequestMapping("create-union-enterprise")
    public RestResponse createUnionEnterprise(String authCode, Integer authCodeId, String enterpriseName) {
        ServiceResponse authCodeResult = authCodeApiService.validateAuthCode(authCodeId, authCode);
        if (!authCodeResult.isSuccess()) {
            return RestResponse.failure(authCodeResult.getCode(), authCodeResult.getMessage());
        }
        EnterpriseDTO unionEnterprise = checkEnterpriseByEnterpriseName(enterpriseName);
        if (unionEnterprise == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "联合商户不存在或者是自有商户");
        }

        Integer ownEnterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Integer unionEnterpriseId = unionEnterprise.getEnterpriseId();
        //查询是否已存在联合商户
        UnionEnterpriseAuthDTO unionEnterpriseAuthDTO = getUnionEnterpriseAuth(ownEnterpriseId, unionEnterpriseId);
        if (unionEnterpriseAuthDTO != null) {
            //已存在，则不需要记录操作日志
            return RestResponse.success();
        } else {
            return OperationResultUtils.operationResult(
                    unionEnterpriseAuthApiService.saveUnionEnterprise(ownEnterpriseId, unionEnterpriseId),
                    OperationResultUtils.LOG_NEW + "联合商户",
                    unionEnterprise.getEnterpriseName());
        }
    }

    /**
     * 资源组
     * @return
     */
    @RequestMapping("list-resource-group")
    public RestResponse listResourceGroup() {
        ServiceResponse<List<ResourceGroupDTO>> response = resourceGroupApiService.listResourceGroup(null,
                UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(response, UnionResourceGroupVO.class);
    }

    /**
     * 受审组
     * @return
     */
    @RequestMapping("list-audited-group")
    public RestResponse listAuditedGroup() {
        ServiceResponse<List<AuditedGroupDTO>> response = auditedGroupApiService.listAllAuditedGroup(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(response, UnionAuditedGroupVO.class);
    }

    /**
     * 账号分组
     * @return
     */
    @RequestMapping("list-account-group")
    public RestResponse listAccountGroup() {
        ServiceResponse<List<AccountGroupDTO>> response = accountGroupApiService.listNoMemberByEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(response, UnionAccountGroupVO.class);
    }

    /**
     * 资源授权
     * @param unionEnterpriseAuthResQO
     * @return
     */
    @RequestMapping("auth-resource")
    public RestResponse authResource(UnionEnterpriseAuthResQO unionEnterpriseAuthResQO) {
        //资源授权
        UnionEnterpriseAuthResDTO dto = EntityUtil.changeEntityNew(UnionEnterpriseAuthResDTO.class, unionEnterpriseAuthResQO);
        dto.setOwnEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        dto.setUnionEnterpriseId(unionEnterpriseAuthResQO.getEnterpriseId());
        return OperationResultUtils.operationResult(
                unionEnterpriseAuthApiService.authResource(dto),
                "资源授权联合",
                getEnterpriseById(dto.getUnionEnterpriseId()).getEnterpriseName());
    }

    @RequestMapping("get-resource-detail")
    public RestResponse getResourceDetail(Integer enterpriseId) {
        ServiceResponse<UnionEnterpriseAuthDTO> response = unionEnterpriseAuthApiService
                .getUnionEnterpriseAuth(UserDetailUtils.getUserDetail().getEnterpriseId(), enterpriseId);
        if (response.isSuccess()) {
            if (response.getResult().getResourceGroupId() == null) {
                return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "还未资源联合");
            }
        }
        return ResultControllerUtils.commonResultOne(response, UnionEnterpriseResourceDetailVO.class);
    }

    /**
     * 解除资源
     * @param enterpriseId 联合商户
     * @param relieveRemark 解除理由
     * @return
     */
    @RequestMapping("relieve-resource")
    public RestResponse relieveResource(Integer enterpriseId, String relieveRemark) {
        //解除资源
        return OperationResultUtils.operationResult(
                unionEnterpriseAuthApiService.relieveUnion(UserDetailUtils.getUserDetail().getEnterpriseId(), enterpriseId, relieveRemark),
                "解除联合",
                getEnterpriseById(enterpriseId).getEnterpriseName());
    }

    /**
     * 联合商户列表
     * @param enterpriseName
     * @param authStatus 资源授权状态
     * @return
     */
    @RequestMapping("list-union-enterprise")
    public RestResponse listUnionEnterprise(String enterpriseName, Integer authStatus) {
        return ResultControllerUtils.commonResult(unionEnterpriseAuthApiService
                        .listUnionEnterpriseAuth(UserDetailUtils.getUserDetail().getEnterpriseId(), enterpriseName, authStatus),
                UnionEnterpriseListVO.class);
    }

    @RequestMapping("resource-reset-auth")
    public RestResponse resourceResetAuth(Integer enterpriseId, Integer resourceType) {
        //重新授权
        return ResultControllerUtils.commonResult(unionEnterpriseAuthApiService
                .resetAuthResource(UserDetailUtils.getUserDetail().getEnterpriseId(), enterpriseId, resourceType));
    }

    @RequestMapping("list-app")
    public RestResponse listApp() {
        ServiceResponse<List<ApplicationDTO>> appListResponse = applicationApiService.listApplicationAndServeProp();
        return ResultControllerUtils.commonResult(appListResponse, UnionAppVO.class);
    }

    @RequestMapping("auth-app-resource")
    public RestResponse authAppResource(@RequestBody UnionEnterpriseAuthAppQO unionEnterpriseAuthAppQO) {
        //todo starrynight app资源授权
        return RestResponse.success();
    }

    @RequestMapping("get-app-resource-detail")
    public RestResponse getAppResourceDetail(Integer unionEnterpriseAuthAppId) {
        //todo starrynight
        return RestResponse.success(new UnionEnterpriseAppResourceDetailVO());
    }

    @RequestMapping("user-area")
    public RestResponse userArea(Integer enterpriseId) {
        //todo starrynight 资源组的用户域数据
        Map<String, String[]> result = new HashMap<>();

        return RestResponse.success(result);
    }

    @RequestMapping("list-app-resource")
    public RestResponse listAppResource(Integer enterpriseId) {
        //todo starrynight

        Page<UnionAppResourceListVO> page = new Page<>();
        return RestResponse.success(page);
    }


    /**
     * 自有商户列表
     * @return
     */
    @RequestMapping("list-own-enterprise")
    public RestResponse listOwnEnterprise() {
        //todo starrynight

        List<OwnEnterpriseListVO> page = new ArrayList<>();
        return RestResponse.success(page);
    }

    @RequestMapping("get-own-enterprise-resource-detail")
    public RestResponse getOwnEnterpriseResourceDetail(Integer ownEnterpriseId) {
        //todo starrynight
        return RestResponse.success(new OwnEnterpriseResourceDetailVO());
    }

    @RequestMapping("get-own-enterprise-app-resource-detail")
    public RestResponse getOwnEnterpriseAppResourceDetail(Integer ownEnterpriseId) {
        //todo starrynight
        List<OwnEnterpriseAppResourceDetailVO> list = new ArrayList<>();
        return RestResponse.success(list);
    }

    @RequestMapping("list-operation-log")
    public RestResponse listOperationLog(Integer ownEnterpriseId, String logContent, String resourceTypeName, String startTime, String endTime) {
        //todo starrynight
        List<OwnEnterpriseOperationLogVO> list = new ArrayList<>();
        return RestResponse.success(list);
    }

    @RequestMapping("get-last-union-info")
    public RestResponse getLastUnionInfo() {
        //todo starrynight
        return RestResponse.success(new OwnEnterpriseUnionInfoVO());
    }


    /**
     * 检查是否存在联合商户
     * @param enterpriseName
     * @return null失败
     */
    private EnterpriseDTO checkEnterpriseByEnterpriseName(String enterpriseName) {
        ServiceResponse<EnterpriseDTO> response = enterpriseApiService.getEnterpriseByName(enterpriseName);
        if (response.isSuccess()) {
            EnterpriseDTO enterpriseDTO = response.getResult();
            if (enterpriseDTO != null) {
                if (enterpriseDTO.getEnterpriseId().equals(UserDetailUtils.getUserDetail().getEnterpriseId())) {
                    return null;
                }
                return enterpriseDTO;
            }
        }
        return null;
    }

    /**
     * 查询联合商户
     * @param ownEnterpriseId
     * @param unionEnterpriseId
     * @return
     */
    private UnionEnterpriseAuthDTO getUnionEnterpriseAuth(Integer ownEnterpriseId, Integer unionEnterpriseId) {
        ServiceResponse<UnionEnterpriseAuthDTO> unionResponse = unionEnterpriseAuthApiService
                .getUnionEnterpriseAuth(ownEnterpriseId, unionEnterpriseId);
        if (unionResponse.isSuccess()) {
            return unionResponse.getResult();
        }
        return null;
    }

    private EnterpriseDTO getEnterpriseById(Integer enterpriseId) {
        //查询联合商户信息
        ServiceResponse<EnterpriseDTO> enterpriseResponse = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!enterpriseResponse.isSuccess()) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "联合商户不存在");
        }
        return enterpriseResponse.getResult();
    }

}
